function TASKSWITCH_design1_task(subject_code, button_set)
%Task Switching Task
%Programmed by: Ajay B. Satpute
%10/22/2008

if button_set == 1
    left_color = 'red';
    left_shape = 'triangle';
    right_color = 'green';
    right_shape = 'circle';
elseif button_set == 2
    left_color = 'red';
    left_shape = 'circle';
    right_color = 'green';
    right_shape = 'triangle';   
elseif button_set == 3
    left_color = 'green';
    left_shape = 'triangle';
    right_color = 'red';
    right_shape = 'circle'; 
elseif button_set == 4
    left_color = 'green';
    left_shape = 'circle';
    right_color = 'red';
    right_shape = 'triangle';     
end
    
%CHECK INPUTS
if ~exist('subject_code'),
    error(sprintf('\n\nPlease enter a subject code.\ne.g.\n>>TASKSWITCH(12)\n'));
end
%1 BE IN THE RIGHT DIRECTORY
    try 
        cd ~/Applications/MATLAB_R2007b/tasks/TASKSWITCH
    catch
        fprintf('USING DIRECTORY: %s\n',pwd);
        if exist('data') ~= 7,         
            error('Data folder does not exist.\nPlease make the directory ''data'' in %s.\nAnd then rerun the function.',pwd);
        end
    end
    
%1.1 CREATE LOGFILE, STIMFILE, STIMPIC FOR SAVING
    logfile = sprintf('data/TS_%d.mat',subject_code);
    if exist(logfile,'file'),
        error(['A datafile of name ' logfile ' already exists!\n']);
    end

    
%1.2 SETUP EXPERIMENT VARIABLES AND SUBJECT DATAFILE
    %counterbalancing=1;  %1= 'stimulation' cue, 2= 'safe' cue 
    flip_counter = 1; %holds flipping index for onsets of trial sub-parts (probe, target, ITI).
        
    disp('Choose a device for subject responses')
    input_device = hid_probe;
    fprintf('\n\n\n');
    disp('Choose a device for experimenter responses')
    experimenter_device = hid_probe;

    
%1.3 LOAD UP (a cell of size 65 that has the trial order and
%info in it.
    try
        load efficient_design_jitter.mat; %loads up structure design 
        %[isidur,cue,stimdur,stim] = textread('ts1.txt','%d%s%d%s');

    catch
        error('Could not load trial information: ts1.txt.\n');
    end
    
    for i = 1:length(design.trialinfo(:,3)),
        if design.trialinfo(i,3) == 1,
        stimrecode{i} = 'redtriangle';
        end
        if design.trialinfo(i,3) == 2,
        stimrecode{i} = 'redcircle';
        end
        if design.trialinfo(i,3) == 3,
        stimrecode{i} = 'greentriangle';
        end
        if design.trialinfo(i,3) == 4,
        stimrecode{i} = 'greencircle';
        end
    end
    
%1.4 Set up trial, block, onset, display images.
    ntrials = length(design.trialinfo); %the number of trials
        
    %Obtain onsets matrix- row refers to trial onset, col 1 = cue onset, col 2 =
    %stim onset.    
    onsets = design.trialinfo(:,6:8);
    cue = design.cue;
    
    %Load images
    for i = 1:4,
        fname = sprintf('%d.bmp',i);
        temp_image = imread(fname,'BMP');
        stim_images{i} = temp_image;
    end  
        
%1.5 Set up response recording variable and file.
    
%1.6 Set up response collection, keys, and devices. 
    [keyIsDown, endrt, KeyCode]=KbCheck; %initialize keypress. 
        %KbCheck is slow to load the first time, but stays loaded afterwards since it's a MEX file. 
        %Hence it should be loaded prior to experimental timing begins.
        %KbCheck is also better than GetChar, which is slow and requires that a
        %character is loaded rather than just any key.

    trigger_key = KbName('!');
        
%1.7. Setup screens for stimulus display and obtain screen parameters such as size.

exp_screen=max(Screen('Screens')); %get screen numbers of which max is used for the initial OpenWindow command.
try
    HideCursor;
    windowptr = Screen('OpenWindow',exp_screen); %opens the screen- I used only a single buffer cause I don't know why two buffers helps
    %grayLevel = 120; %for setting background color
    Screen('FillRect',windowptr,0); %sets background color to black
    [wWidth, wHeight] = Screen('WindowSize',windowptr);
    xcenter=wWidth/2;
    ycenter=wHeight/2;    
  
    Screen('TextSize', windowptr, 24); %Set textsize
    Screen('TextFont',windowptr, 'Arial'); %Set text font
    Screen('TextColor',windowptr,255); %Set text color
    Screen('DrawText', windowptr, 'When cued with "S" or "Shape" respond to the shape of the image.', 50, 75);
    Screen('DrawText', windowptr, sprintf('FIRST button = %s', left_shape), 80, 125);
    Screen('DrawText', windowptr, sprintf('SECOND button = %s', right_shape), 80, 175);
    Screen('DrawText', windowptr, 'When cued with "C" or "Color" respond to the color of the image.', 50, 275);
    Screen('DrawText', windowptr, sprintf('FIRST button = %s', left_color), 80, 325);
    Screen('DrawText', windowptr, sprintf('SECOND button = %s', right_color), 80, 375);
    Screen('DrawText', windowptr, 'Respond as quickly and as accurately as possible.', 50, 475);
    Screen('Flip',windowptr); %second grey screen [VBLTimestamp, startrt, FlipTimestamp, missed] 
catch
    fprintf('Experiment Failure: Screen initializing error.\n');
    Screen('CloseAll');
    ShowCursor;
end

%size_factor = .90*min([wWidth wHeight]); %SIZE currently takes 90% of the minimum of either the width or height as the largest boundary.
%SIZE = .5*size_factor;
SIZE = 150; %This sets up the size factor for the square images. Should be 1/2 the size of the image you want for square images. 
            %The size of the BMP files is 300x300, so I set this to 150.

%Set letter and word positions...
Screen('TextSize', windowptr, 110); %Set textsize
VERT = 50; %vertical position. increase to move words/letters/targets down, decrease to move up. Does not influence fixation.
%horizontal positions are hard coded within each line.

[normBoundsRect] = Screen('TextBounds',windowptr,'+');   
fix_xpos = xcenter-fix(normBoundsRect(3)/2);
fix_ypos = ycenter-fix(normBoundsRect(4)/2);
cue_ypos = ycenter-SIZE-fix(normBoundsRect(4))-10 + VERT;

try
%2 EXPERIMENT BEGINS!
%Starting key press
flip_counter = 1;

%TRIGGER
        trigger = KbName('t');
        secs=KbTriggerWait(KbName('t'),input_device);
      % If using the keyboard, allow any key as input
        noresp=1;
        while noresp,
        [keyIsDown,secs,keyCode] = KbCheck(input_device);
            if keyIsDown & noresp,
                noresp=0;
            end;
        end;
        WaitSecs(0.001);
    % WaitSecs(0.5);  % prevent key spillover--ONLY FOR BEHAV VERSION
    
    
    DisableKeysForKbCheck(KbName('t')); % So trigger is no longer detected

anchor_ghostTRs = GetSecs;
%GHOST TRs
Screen('Flip',windowptr);
anchor = GetSecs; %gets starting time relative to first collected TR (not ghost TR).
%DisableKeysForKbCheck(KbName('z')); %this reenables the 't' key, but temporarily diables the 'z' key to do so. 
                             
for trials = 1:ntrials,  
        %CUE STIMULUS

            [normBoundsRect] = Screen('TextBounds',windowptr,cue{trials});   
            Screen('DrawText', windowptr, cue{trials}, xcenter-fix(normBoundsRect(3)/2), cue_ypos);
 
            while (GetSecs - anchor) < onsets(trials,flip_counter),
                    WaitSecs(.01);
            end %while
            [VBLTimestamp SOT_CUE] = Screen('Flip',windowptr);
            flip_counter = flip_counter + 1;
        %PROBE STIMULUS
            Screen('DrawText', windowptr, cue{trials}, xcenter-fix(normBoundsRect(3)/2), cue_ypos);
            if design.trialinfo(trials,3) == 1,
                Screen('PutImage',windowptr,stim_images{1},[xcenter-SIZE ycenter-SIZE+VERT xcenter+SIZE ycenter+SIZE+VERT]);
            end
            if design.trialinfo(trials,3) == 2,
                Screen('PutImage',windowptr,stim_images{2},[xcenter-SIZE ycenter-SIZE+VERT xcenter+SIZE ycenter+SIZE+VERT]);
            end
            if design.trialinfo(trials,3) == 3,
                Screen('PutImage',windowptr,stim_images{3},[xcenter-SIZE ycenter-SIZE+VERT xcenter+SIZE ycenter+SIZE+VERT]);
            end
            if design.trialinfo(trials,3) == 4,
                Screen('PutImage',windowptr,stim_images{4},[xcenter-SIZE ycenter-SIZE+VERT xcenter+SIZE ycenter+SIZE+VERT]);
            end

            while (GetSecs - anchor) < onsets(trials,flip_counter),
                    WaitSecs(.01);
            end %while
            [VBLTimestamp SOT_PROBE] = Screen('Flip',windowptr);
            flip_counter = flip_counter + 1;

            Screen('DrawText', windowptr, '+', fix_xpos, fix_ypos);
            
            % to collect responses and RTs
            noresp=1;
            while noresp, 
                [keyIsDown,secs,keyCode] = KbCheck(input_device);
                if keyIsDown && noresp,
                     %tmp=KbName(keyCode);
                     tmp = find(keyCode);
                     response=tmp(1);
                     RT=GetSecs-SOT_PROBE;
                     noresp=0;
                end

                if (GetSecs - anchor) > onsets(trials,flip_counter),
                    response = NaN;
                    RT= NaN; 
                    noresp=0;
                end
                WaitSecs(0.01);
            end

            while (GetSecs - anchor) < onsets(trials,flip_counter), %waits any remaining time
                    WaitSecs(.01);
            end %while

            [VBLTimestamp SOT_ITI] = Screen('Flip',windowptr);
            flip_counter = 1; %resets flip_counter for next trial

            rec_task{trials,1} = trials; 
            rec_task{trials,2} = cue{trials}; 
            rec_task{trials,3} = stimrecode{trials}; 
            rec_task{trials,4} = response;
            rec_task{trials,5} = RT;
            rec_task{trials,6} = SOT_CUE-anchor;
            rec_task{trials,7} = SOT_PROBE-anchor;
            rec_task{trials,8} = SOT_ITI-anchor; 
            save(logfile,'rec_task'); %saves the output rec_task as a .mat file
            
end %for trials

catch
    Screen('CloseAll');
    ShowCursor;
    fprintf('Program display error.\n');
    rethrow(lasterror);
    save(logfile,'rec_task', 'button_set');
end

save(logfile,'rec_task', 'button_set'); %saves the output rec_task as a .mat file
Screen('TextSize', windowptr, 60); %Set textsize
[normBoundsRect] = Screen('TextBounds',windowptr,'Run Completed!');   
Screen('DrawText', windowptr, 'Run Completed!', xcenter-fix(normBoundsRect(3)/2), ycenter-fix(normBoundsRect(4)/2));
WaitSecs(8); %Catches response of last trial.
Screen('Flip',windowptr); %second grey screen [VBLTimestamp, startrt, FlipTimestamp, missed] 
WaitSecs(3);
Screen('CloseAll');
ShowCursor
